"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for bcmcmd phy control xe/ge/ceXX
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd phy control"
aliases = ['bcm phy control']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    '''
root@qct-ly8-04:~# bcm phy control xe13
Current PHY control settings of xe13 ->
Preemphasis              = 0x3f0
DriverCurrent            = 0x3f0
PreDriverCurrent         = 0x3f0
LinkTraining Enable      = False
Rx LOS (s/w) enable         - 1
    '''
    '''
    Tomahawk:
Current PHY control settings of xe1 ->
Preemphasis              = 0xc6400
DriverCurrent            = 0x8
DFE ENable               = True
LP DFE ENable            = False
BR DFE ENable            = False
LinkTraining Enable      = False
Interface                = 0x28

    Helix-4  ge48
Current PHY control settings of ge48 ->
WAN                      = 0x0
WAN                      = 0x0
Preemphasis              = 0x0
DriverCurrent            = 0x0
PreDriverCurrent         = 0x0
DFE ENable               = False
LP DFE ENable            = False
BR DFE ENable            = False
LinkTraining Enable      = False
EqualizerBoost           = 0x0
Interface                = 0x0
InterfaceMAX             = 0x0
Rx LOS (s/w) enable         - 0

       Fixed Copper ports:
Current PHY control settings of ge9 ->
Preemphasis              = 0x0


    '''
    values = ['preemphasis', 'drivercurrent', 'predrivercurrent',
              'dfe_enable', 'lp_dfe_enable', 'br_dfe_enable',
              'linktraining_enable', 'rx_los_enable', 'interface']

    split_pattern = re.compile('Current\sPHY\scontrol\ssettings\sof\s(?P<phy>[xgcehg0-9]+)\s->')

    phy_pattern = re.compile(r'''^
    (\s*WAN\s+=\s+(?P<wan1>[0-9xa-f]+))?
    (\s*WAN\s+=\s+(?P<wan2>[0-9xa-f]+))?
    \s+Preemphasis\s+=\s+(?P<preemphasis>[0-9xa-f]+)
    (\s+DriverCurrent\s+=\s+(?P<drivercurrent>[0-9xa-f]+))?
    (\s+PreDriverCurrent\s+=\s+(?P<predrivercurrent>[0-9xa-f]+))?
    (\s+DFE\sENable\s+=\s+(?P<dfe_enable>[a-zA-Z]+))?
    (\s+LP\sDFE\sENable\s+=\s+(?P<lp_dfe_enable>[a-zA-Z]+))?
    (\s+BR\sDFE\sENable\s+=\s+(?P<br_dfe_enable>[a-zA-Z]+))?
    (\s+LinkTraining\sEnable\s+=\s+(?P<linktraining_enable>[a-zA-Z]+))?
    (\s+EqualizerBoost\s+=\s+(?P<equalizer_boost>[0-9xa-f]+))?
    (\s+Interface\s+=\s+(?P<interface>[0-9xa-f]+))?
    (\s+InterfaceMAX\s+=\s+(?P<interface_max>[0-9xa-f]+))?
    (\s+Rx\sLOS\s\(s/w\)\senable\s+(?P<rx_los_enable>[- 0-9]+))?
    \s*$''', re.X)

    parsed_dct = {}
    # split the output on the first line of each phy group, collecting the phy along the way.
    # The list will look like [ '', phy0, output0, phy1, output1 ]
    # drop the first item and iterate on the rest

    # For Maverick T platforms, strip off failed lines:
    # <nounit>:portmod_port_ext_phy_control_get: Feature unavailable
    output1 = ''
    for line in output.split('\n'):
        if 'Feature' in line:
            continue
        output1 = '\n'.join([output1, line])

    phy_control_list = split_pattern.split(output1)[1:]
    for i in xrange(0, len(phy_control_list), 2):
        phy = phy_control_list[i]
        debug('Phy is: {}'.format(phy))
        phy_out = phy_control_list[i+1]

        parsed_dct[phy] = {}
        match = phy_pattern.match(phy_out)
        if match:
            debug('Match: ' + pformat(match.group(0)))
            parsed_dct[phy] = match.groupdict()

        else:
            error('No {} parser match: {} on phy: {}'.format(name, phy_out, phy))
            return None

    return parsed_dct



    # for line in output.split('\n'):
    #     if re.search('Current\sPHY\scontrol\ssettings\sof', line):
    #         continue
    #
    #     if re.match('Preemphasis\s+=\s+(?P<preemphasis>[0-9xa-f]+)', line):
    #         debug('Preemphasis: {}'.format(match.group('preemphasis')))
    #         bcm_phy_control_dct['preemphasis'] = match.group('preemphasis')
    #
    #     elif re.match('DriverCurrent\s+=(?P<drivercurrent>[0-9xa-f]+)', line):
    #         debug('DriverCurrent: {}'.format(match.group('drivercurrent')))
    #         bcm_phy_control_dct['drivercurrent'] = match.group('drivercurrent')
    #
    #     elif re.match('PreDriverCurrent\s+=(?P<predrivercurrent>[0-9xa-f]+)', line):
    #         debug('PreDriverCurrent: {}'.format(match.group('predrivercurrent')))
    #         bcm_phy_control_dct['predrivercurrent'] = match.group('predrivercurrent')
    #
    #     elif re.match('LinkTraining\sEnable\s+=(?P<linktrainingenable>[a-z]+)', line):
    #         debug('LinkTrainingEnable: {}'.format(match.group('linktrainingenable')))
    #         bcm_phy_control_dct['linktrainingenable'] = match.group('linktrainingenable')
    #
    #     elif re.match('Rx\sLOS\s\(s/w\)\senable\s+(?P<rxlosenable>[- 0-9]+)', line):
    #         debug('RxLOSEnable: {}'.format(match.group('rxlosenable')))
    #         bcm_phy_control_dct['rxlosenable'] = match.group('rxlosenable')
    #
    #     else:
    #         error('Parser Bug: No Match found on line:')
    #         error(line)
    #         return None
    #
    # return bcm_phy_control_dct

