"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import logging
import re
from pprint import pformat

"""
bcmcmd portstat parser
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd portstat"
aliases = ["bcm portstat", "bcmcmd ps", "bcm ps"]


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    #        ena/    speed/ link auto    STP                  lrn  inter   max  loop
    #  port  link    duplex scan neg?   state   pause  discrd ops   face frame  back
    #   xe0  down   40G  FD   SW  No   Disable          None    F  XGMII  1518
    #   xe1  down   40G  FD   SW  No   Disable          None    F  XGMII  1518

    # SDK 6.4.11 :
    #           ena/    speed/ link auto    STP                  lrn  inter   max  loop
    #     port  link    duplex scan neg?   state   pause  discrd ops   face frame  back
    # xe0(  1)  up     10G  FD   SW  No   Disable          None    F    SFI  1518
    # xe1(  2)  up     10G  FD   SW  No   Disable          None    F    SFI  1518
    # xe2(  3)  down   10G  FD   SW  No   Disable          None    F    SFI  1518
    # xe68( 69)  down   10G  FD   SW  No   Disable          None    F    XFI  1518

    # 5812, sdk 6.5.7: TD2+ when autoneg enabled, speed/duplex column is '-'
    # xe0(  1)  down     -           SW  Yes  Disable          None   FA    XFI  9018
    # xe6( 34)  up     1   10G  FD   SW  No   Disable          None    F    XFI  1518    No
    #    ge48( 49)  !ena    1G  FD   SW  No   Disable  TX RX   None   FA   GMII 16356

    # SDK 6.5.14, 4.0 w/ autoneg
    # xe9( 92)  down   2   50G  FD   SW  No   Disable          None    F    CR2  1518    No

    pattern = re.compile(r"""
        \s+(?P<phy>[cxg]e[0-9]+)
           (?P<log_port>\([0-9 ]+\))?
        \s+(?P<linkstate>[a-z\!]+)
       (\s+(?P<lns>[124]))?
        \s+(?P<speed>[0-9GM\-]+)
       (\s+(?P<duplex>[FHD]+))?
        \s+(?P<link_scan>[SW]+)
        \s+(?P<autoneg>[NoYes]+)
        \s+(?P<stp_state>[DisEnableForward]+)
       (\s+(?P<pause>[TRX ]+))?
        \s+(?P<discard>[None]+)
        \s+(?P<learn_ops>[A-Z]+)
        \s+(?P<cable>[A-Z24]+)
        \s+(?P<mtu>[0-9]+)
       (\s+(?P<loopback>[YesNo]+))?
    """, re.X)

    bcm_portstat_dct = {}
    for line in output.split('\n'):
        debug('BCM-PS:{}'.format(line))
        if re.search('port', line):
            continue

        if re.search('STP', line):
            continue

        if not re.search('[cxg]e[0-9]', line):
            continue

        match = pattern.match(line)
        if match:
            debug('Match: ' + pformat(match.group(0)))
            phy = match.group('phy')  # 2
            bcm_portstat_dct[phy] = match.groupdict()
            bcm_portstat_dct[phy].pop('phy', 0)
            # Handle autoneg scenario on some boxes (TD2+) where speed/duplex is '-'/blank
            if bcm_portstat_dct[phy]['speed'] == '-':
                bcm_portstat_dct[phy]['speed'] = None

        else:
            error('Parser bug: No Match found on line:')
            error(line)
            return None

    return bcm_portstat_dct


