"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for bcmcmd phy dumpall c22
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "bcmcmd show portmap"
aliases = ['bcm show portmap']


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """
    hdr_pattern_3 = re.compile(r'''^
        \s*pipe
        \s+logical
        \s+physical
       (\s+idb)?
        \s+mmu
        \s+ucast_Qbase/Numq
        \s+mcast_Qbase/Numq
        (\s+endpoint_queue/Numq)?
        (\s+service_queue/Numq)?
        \s*$
    ''', re.X)

    hdr_pattern_3_helix = re.compile(r'''^
        \s*pm\(pgw\)
        \s+logical
        \s+physical
        \s+lanes
        \s+speed_max
        \s+mmu
        \s+ucq/Numq
        \s+mcq/Numq
        \s*$
    ''', re.X)

    hdr_pattern_3_helix_v2 = re.compile(r'''^
        \s*pm\(pgw\)
        \s+logical
        \s+physical
        \s+idb
        \s+lanes
        \s+speed_max
        \s+mmu
        \s+ucq/Numq
        \s+mcq/Numq
        \s*$
    ''', re.X)


    hdr_pattern_2 = re.compile(r'''^
        \s*logical
        \s+physical
        \s+mmu
        \s+ucast\s+Qbase
        \s+mcast\s+Qbase
        \s*$
    ''', re.X)

    hdr_pattern_tomahawk3 = re.compile(r'''^
        \s*pipe
        \s+logical
        \s+physical
        \s+MMU\(local\)
        \s+MMU\(chip\)
        \s+ucast_Qbase/Numq
        \s+mcast_Qbase/Numq
        \s*$
    ''', re.X)

    # CL3
    '''
             pipe   logical  physical    mmu   ucast_Qbase/Numq  mcast_Qbase/Numq
      cpu0      0     0         0        52         0/0               2000/44
       xe0      0     1        13         0         0/12              1480/10
       xe1      0     2        14         1        12/12              1490/10
       xe2      0     3        15         2        24/12              1500/10

    # Hurricane2 in SDK 6.5.7
    /usr/lib/cumulus/bcmcmd 0: show portmap'
              pipe   logical  physical    mmu   ucast_Qbase/Numq  mcast_Qbase/Numq
      cpu0      0     0         0        52         0/0               2000/44  
       xe0      0     1        25         3        32/12              1510/10  
       xe1      0     2        26         4        44/12              1520/10  
    '''

    data_pattern_3 = re.compile(r'''^
        \s*(?P<phy>[a-z0-9]+)
        \s+(?P<pipe>\d+)
        \s+(?P<logical>\d+)
        \s+(?P<physical>\d+)
        \s+(?P<idb>\d+)?
        \s+(?P<mmu>\d+)
        \s+(?P<ucast_Qbase>\d+)/
           (?P<ucast_NumQ>\d+)
        \s+(?P<mcast_Qbase>\d+)/
           (?P<mcast_NumQ>\d+)
        (\s+(?P<endpoint_queue>\d+)/
            (?P<endpoint_queue_NumQ>\d+))?
        (\s+(?P<service_queue>\d+)/
            (?P<service_queue_NumQ>\d+))?
        \s*$
    ''', re.X)

    # Helix4
    '''
    root@act-4610-54p-02:~# /usr/lib/cumulus/bcmcmd show portmap
             pm(pgw)  logical  physical   lanes   speed_max   mmu   ucq/Numq    mcq/Numq
      cpu0    0(0)       0         0       -1            0    59       0/0       512/45
    '''

    # Helix4 and Hurricane2 output in SDK 6.5.12
    '''
2018-01-31 08:35:16,522: [cumulus@qct-ly4r-04:22] sudo: ('/usr/lib/cumulus/bcmcmd show portmap',)
              pm(pgw)  logical  physical   idb lanes   speed_max   mmu   ucq/Numq    mcq/Numq
      cpu0    0(0)       0         0         0  0            0     0       0/0         0/0   
       ge0    0(0)       2         9         0  1         1000     2       0/0         0/0   
    '''


    data_pattern_3_helix = re.compile(r'''^
        \s*(?P<phy>[a-z0-9]+)
        \s+(?P<pm>\d+)
           (?P<pgw>\(\d+\))
        \s+(?P<logical>\d+)
        \s+(?P<physical>\d+)
        \s+(?P<lanes>[-0-9]+)
        \s+(?P<speed_max>[-0-9]+)
        \s+(?P<mmu>\d+)
        \s+(?P<ucast_Qbase>\d+)/
           (?P<ucast_NumQ>\d+)
        \s+(?P<mcast_Qbase>\d+)/
           (?P<mcast_NumQ>\d+)
        \s*$
    ''', re.X)

    data_pattern_3_helix_v2 = re.compile(r'''^
        \s*(?P<phy>[a-z0-9]+)
        \s+(?P<pm>\d+)
           (?P<pgw>\(\d+\))
        \s+(?P<logical>\d+)
        \s+(?P<physical>\d+)
        \s+(?P<idb>[-0-9]+)
        \s+(?P<lanes>[-0-9]+)
        \s+(?P<speed_max>[-0-9]+)
        \s+(?P<mmu>\d+)
        \s+(?P<ucast_Qbase>\d+)/
           (?P<ucast_NumQ>\d+)
        \s+(?P<mcast_Qbase>\d+)/
           (?P<mcast_NumQ>\d+)
        \s*$
    ''', re.X)


    # CL2
    '''
             logical  physical    mmu   ucast Qbase  mcast Qbase
      cpu0      0         0         0         0            0
       xe0      1        13         3        48           58
       xe1      2        14         7       127          137
       xe2      3        15        11       142          152
    '''

    data_pattern_2 = re.compile(r'''^
        \s*(?P<phy>[a-z0-9]+)
        \s+(?P<logical>\d+)
        \s+(?P<physical>\d+)
        \s+(?P<mmu>\d+)
        \s+(?P<ucast_Qbase>\d+)
        \s+(?P<mcast_Qbase>\d+)
        \s*$
     ''', re.X)

    data_pattern_tomahawk3 = re.compile(r'''^
       \s*(?P<phy>[a-z0-9]+)
        \s+(?P<pipe>\d+)
        \s+(?P<logical>\d+)
        \s+(?P<physical>\d+)
        \s+(?P<mmu_local>\d+)
        \s+(?P<mmu_chip>\d+)
        \s+(?P<ucast_Qbase>\d+)/
           (?P<ucast_NumQ>\d+)
        \s+(?P<mcast_Qbase>\d+)/
           (?P<mcast_NumQ>\d+)
        \s*$
    ''', re.X)

    parsed_dct = {}
    first_line = output.split('\n')[0]
    debug('First line:{}'.format(first_line))
    if hdr_pattern_2.match(first_line):
        debug('Output is from a CL2 switch')
        data_regex = data_pattern_2
    elif hdr_pattern_3.match(first_line):
        debug('Output is from a CL3 switch')
        data_regex = data_pattern_3
    elif hdr_pattern_3_helix.match(first_line):
        debug('Output is from a Helix switch')
        data_regex = data_pattern_3_helix
    elif hdr_pattern_3_helix_v2.match(first_line):
        debug('Output is from a Helix or Hurricane switch in SDK 6.5.12+')
        data_regex = data_pattern_3_helix_v2
    elif hdr_pattern_tomahawk3.match(first_line):
        debug('Output is from a tomahawk3 switch')
        data_regex = data_pattern_tomahawk3

    else:
        error('{}: No match of a header pattern for first line:{}'.format(name, first_line))
        return None

    # Skip header line
    for line in output.split('\n')[1:]:
        debug('Line: {}'.format(line))
        match = data_regex.match(line)

        if match:
            debug('Match: ' + pformat(match.group(0)))
            phy = match.group('phy')
            parsed_dct[phy] = match.groupdict()
            parsed_dct[phy].pop('phy', 0)
        else:
            error('Parser Bug: No Match found on line:')
            error(line)
            return None

    return parsed_dct
