"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""

import re
import logging
from pprint import pformat

"""
Parser for /usr/lib/cumulus/mlxcmd ports info
Create a dictionary with all the port info for each port in the table.
"""

this_log = logging.getLogger(__name__)
debug    = this_log.debug
info     = this_log.info
warn     = this_log.warning
error    = this_log.error

name = "mlxcmd ports info"
aliases = ['mlx ports info']

'''
Port ID Hex Port ID     Port    Status  Speed   FEC
65792   0x00010100      swp33   Down\1522       0 Gbps  Unknown
66048   0x00010200      swp34   Down\1522       0 Gbps  Unknown
69632   0x00011000      swp48   Up\1522 10 Gbps Auto
70400   0x00011300      swp50   Up\1522 40 Gbps Auto
71936   0x00011900      swp53   Up\1522 100 Gbps        Auto
81152   0x00013d00      swp1    Up\1522 25 Gbps Auto
'''


def parse(command, output):
    """
    Return a dictionary or list containing the values parsed from the output
    If parse failed, return None
    """

    hdr_pattern = re.compile(r'''Port\s+ID\s+Hex\s+Port\s+ID\s+Port\s+Status\s+Speed\s+FEC''')

    table_pattern = re.compile(r'''
     \s*(?P<port_id>\w+)
     \s+(?P<hex_port_id>0x[a-f0-9]+)
     \s+(?P<port>[swp0-9]+)
     \s+(?P<linkstate>Up|Down)\\
        (?P<mtu>\d+)
     \s+(?P<speed>\d+\s+Gbps)
     \s+(?P<fec>\w+)
    ''', re.X)

    parsed_dct = {}
    # or if matching one line at a time:
    for line in output.split('\n'):
        if hdr_pattern.match(line):
            continue

        match = table_pattern.match(line)
        if match:
            debug('Match: ' + pformat(match.group(0)))
            port = match.group('port')
            # For each possible match group, create the dictionary keys = values
            parsed_dct[port] = match.groupdict()
            # Since extracting the 'port' key and using it at a higher level, we remove it here:
            parsed_dct[port].pop('port', 0)
            # Change linkstate to be all lowercase to match bcm portstat output
            parsed_dct[port]['linkstate'] = parsed_dct[port]['linkstate'].lower()
            # change 'X Gbps' to 'XG'
            parsed_dct[port]['speed'] = re.sub(r' ([GM])bps', r'\1', parsed_dct[port]['speed'])

        else:
            error('No {} parser match: {}'.format(name, output))
            return None

    return parsed_dct
