"""
Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved

This software is subject to the Cumulus Networks End User License Agreement available
at the following locations:

Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
Cumulus Linux systems: /usr/share/cumulus/EULA.txt
"""


import os
import pkgutil
import logging

import cl_parsers.parsers as parsers

_registry = dict()

this_log = logging.getLogger(__name__)
debug = this_log.debug
info = this_log.info
warn = this_log.warning
error = this_log.error

# ch = logging.StreamHandler()
# ch.setLevel(logging.INFO)
# this_log.addHandler(ch)


class _NullParser(object):

    def __init__(self):
        self.name = "null"

    def parse(self, command, output):
        return None

NullParser = _NullParser()


def get_parser(command):
    """
    Searches the registry for a command and returns the longest match.
    Supports recursive lookups.  e.g. for 'cat filename', the cat parser calls get_parser with a
     recursive lookups to call the 'filename' parser
    Removes any path information, expecting a filename will only have one unique parser.  If not,
    a if statement with a recursive lookup within a parser could handle multiple options,
    """

    global _registry
    debug('Registry: {}'.format(_registry))
    assert isinstance(command, str) or isinstance(command, unicode), "Command must be a string"
    # debug('Original command: {}'.format(command))
    to_match = command
    debug('Cmd To Match: {}'.format(to_match))

    match = ("", 0)

    for temp in _registry.keys():
        args = temp.strip().split(' ')
        tm = to_match.strip().split(' ')
        tm[0] = os.path.basename(tm[0])
        idx = 0
        score = 0.0
        for part in tm:
            if args[idx] in part:
                score += float(len(args[idx])) / float(len(part))
                idx += 1
                if idx == len(args): break
        if idx == len(args):
            if score > match[1]:
                match = (temp, score)

    if match[1] >= 1:
        debug('Parser matches: {}'.format(_registry[match[0]]))
        return _registry[match[0]]
                
    warn('No parser found for {}'.format(command))
    return NullParser


def add_parser(module):
    '''
    Adds the module to the registry using its name variable or converts the modules's __name__ to the name by
    changing the underscores to spaces.
    '''
    global _registry
    debug('Adding module: {} to registry'.format(module))
    _registry[getattr(module, 'name', " ".join(module.__name__.rpartition('.')[2].split("_")))] = module


def parsers_iterator(path, submodules=[]):
    '''
    Finds all parsers int cl_parsers.parsers directory and returns an iterator of them.
    '''
    for loader, parser, is_package in pkgutil.walk_packages(path):
        if is_package:
            new_path = os.path.join(path[0], parser)
            if not os.path.exists(new_path):
                raise Exception("Not found: " + new_path)
            for p in parsers_iterator([new_path], submodules + [parser]):
                yield p
        else:
            parser_name = ".".join(['cl_parsers', 'parsers'] + submodules + [parser])
            p = __import__(parser_name)
            for part in parser_name.split('.')[1:]:
                p = getattr(p, part)
            yield p


def add_all_parsers():
    '''
    Finds all parsers int cl_parsers.parsers directory and adds them to the registry.
    '''
    for parser in parsers_iterator(parsers.__path__):
        add_parser(parser)
    debug(_registry)

add_all_parsers()

