#!/usr/bin/env python

# Copyright (C) 2019 Cumulus Networks, Inc. all rights reserved
#
# This software is subject to the Cumulus Networks End User License Agreement available
# at the following locations:
#
# Internet: https://cumulusnetworks.com/downloads/eula/latest/view/
# Cumulus Linux systems: /usr/share/cumulus/EULA.txt

import json
import logging
import re
import sys
import time
import subprocess
import socket
import ConfigParser
import StringIO
import os.path
import re
from os import path
from pprint import pformat

from cl_parsers.parse import parse
from cumulus.platforms import probe
from portwd_utils import SFFCable

from module_qualify.qualify_utils import swp_sort_pattern


this_log = logging.getLogger(__name__)

debug = this_log.debug
info = this_log.info
warn = this_log.warning
error = this_log.error


def critical(mesg):
    this_log.critical(mesg)
    this_log.critical('Exiting abnormally')
    sys.exit()


def debug_test():
    debug('Logging testing of dut_obj submodule')


class Dut(object):

    def __init__(self):
        self.hostname = socket.gethostname().split('.')[0]
        self.platform_object = probe()
        self.asic_vendor = None
        self._get_asic_vendor()
        self.asic = None
        self._get_asic()

        self.ethmode_map = {'40g-cr4':    24,
                            '40g-sr4':    25,
                            '40g-lr4':    26,
                            '10g-cr':     42,
                            '10g-sr':     43,
                            '10g-lr':     44,
                            '10g-lrm':    45,
                            '100g-sr4':   37,
                            '100g-cr4':   38,
                            '100g-lr4':   39,
                            '25g-cr':     31,
                            '25g-sr':     33,
                            '10g-t':      12,
                            '10g-tsr':    12,
                            '50g-cr2':    34,
                            '50g-sr2':    40,
                            '2.5g-t':     47,
                            '5g-t':       48,
                            '1000base-t':  5,
                            '1000base-x': 41,
                            'Zero':        0,
                            }
        self.reverse_ethmode_map = {}
        for ethmode_text in sorted(self.ethmode_map):
            if self.ethmode_map[ethmode_text] not in self.reverse_ethmode_map:
                self.reverse_ethmode_map[str(self.ethmode_map[ethmode_text])] = ethmode_text

    def _get_asic_vendor(self):
        if self.platform_object and self.platform_object.switch and self.platform_object.switch.chip:
            self.asic_vendor = self.platform_object.switch.chip.sw_base
        else:
            warn('ASIC vendor not found on this platform')
            self.asic_vendor = None

    def _get_asic(self):
        if self.platform_object and self.platform_object.chip and self.platform_object.chip.chip_dir_list:
            for platform_dir in self.platform_object.chip.chip_dir_list:
                dir_list = platform_dir.strip().split('/')
                for dir_item in dir_list:
                    if dir_item != '' and self.asic_vendor not in dir_item:
                        self.asic = dir_item
                        return
        warn('ASIC not found on this platform')


def setup_logging(config_file_path=None,
                  log_file_dir='./',
                  quiet=False,
                  default_level=logging.INFO,
                  logformat='%(asctime)s: %(name)20s: %(levelname)s: %(message)s'):
    """
    Setup logging config for a module using a JSON config file specified by config_file_path
      and a log directory target specified by log_file_dir

    The default configuration included in this module configures the following loggers:
      autolib.netobjects for DUT output:  outputs to 'autolib.log' file with level DEBUG
      main module (initial called script): outputs to 'info.log' file with level INFO
      paramiko.transport: paramiko outputs are set to ERROR level to ignore most messages and outputs to 'info.log' file
    a few other handlers are defined for further customization.

    :param config_file_path: the location of a logging config file to load.  Default is the one included in this repo.
    :param log_file_dir: the directory to write the files to.  Default is the present working directory
    :param quiet: if set to True, suppress the initial setup messages describing the location of the config and log files
    :param default_level: In absence of a valid config_file_path file, this sets a basic logger of this level
    :param logformat: The default log message format to use
    
    return the config_dct in case the user wants to reconfigure
    """
    if not config_file_path:
        config_file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'default_log_config.json')
    if not quiet:
        print('log_utils: Reading logging config file from: {}'.format(config_file_path))
    config_file_output = read_local_file(config_file_path)
    if config_file_output:
        config = json.loads(config_file_output)
        config_dct = _insert_logging_path(config, log_file_dir, quiet)
        logging.config.dictConfig(config_dct)
        return config_dct
    else:
        if not quiet:
            print('log_utils: config_file_path did not exist, setting basicConfig, level={}'.format(default_level))

        logging.basicConfig(level=default_level, format=logformat)


def _insert_logging_path(config_dct, log_file_dir, quiet=False):
    """
    Internal function to change the logging config to send the logs to a specified directory
    :param config_dct: Logging Dictionary to modify with log_file_dir
    :param log_file_dir: Logging dir path to insert in log config in order to control log locations
    :param quiet: Set to True to suppress pre-log printed messages
    :return:
    """
    for handler_name in config_dct['handlers']:
        for key in config_dct['handlers'][handler_name]:
            if key == 'filename':
                old_filename = config_dct['handlers'][handler_name][key]
                new_filename = os.path.join(log_file_dir, old_filename)
                config_dct['handlers'][handler_name][key] = new_filename
                if not quiet:
                    print('log_utils: Setting logging path for handler {} to: {}'.format(handler_name, new_filename))

    return config_dct


def debug_logging(modules, this_log=None):
    if not isinstance(modules, list):
        modules = [modules]

    if not this_log:
        this_log = logging.getLogger()

    if 'all' in [modules]:
        for name in sorted(logging.Logger.manager.loggerDict):
            logger_to_change = logging.getLogger(name)
            this_log.info('Logger {} Level: {}. Changing to DEBUG'.format(
                name, logging.getLevelName(logger_to_change.getEffectiveLevel())))
            logger_to_change.setLevel('DEBUG')

    else:
        for module in modules:
            if 'main' in module:
                module = '__main__'
            logger_to_change = logging.getLogger(module)
            this_log.info('Logger {} Level: {}. Changing to DEBUG'.format(
                module, logging.getLevelName(logger_to_change.getEffectiveLevel())))
            logger_to_change.setLevel('DEBUG')
            if 'main' in module:
                this_log.debug('Doc info:')
                this_log.debug(__doc__)


def list_logger_levels(logger_list=logging.Logger.manager.loggerDict):
    if not isinstance(logger_list, list) and not isinstance(logger_list, dict):
        logger_list = [logger_list]

    for logger_to_read in logger_list:
        info('Logger: {} Level: {}'.format(logger_to_read,
                                           logging.getLevelName(logging.getLogger(logger_to_read).getEffectiveLevel())
                                           ))


def shell_out_catch_errors(input_line):
    try:
        output = subprocess.check_output(input_line, shell=True).strip()

    except subprocess.CalledProcessError as e:
        # error executing, collect any output there was
        output = e.output

    return output


def parse_to_dct(dut, cmd, *args, **kwargs):
    out = shell_out_catch_errors(cmd)
    pattern = re.compile(r''' [01]: ''')
    cmd = pattern.sub(' ', cmd)
    return parse(cmd, out)


def read_local_file(filename, mode='rt'):
    try:
        with open(filename, mode) as f:
            return f.read().strip()
    except IOError:
        return None


def find_path(name, path):
    for root, dirs, files in os.walk(path):
        if name in files:
            return root


def dut_obj_install_string(dut, stringname, string_to_attach):
    if not hasattr(dut, stringname):
        setattr(dut, stringname, str())

    debug('Adding string: \"{0}\" to object: \"{1}\"'.format(stringname, dut.hostname))
    setattr(dut, stringname, string_to_attach)


def dut_obj_install_list(dut, listname, list_to_attach):
    if not hasattr(dut, listname):
        setattr(dut, listname, list())

    debug('Adding list \"{0}\" to \"{1}\" object'.format(listname, dut.hostname))
    setattr(dut, listname, list_to_attach)


def dut_obj_install_dict(dut, dictname, dict_to_attach):
    if not hasattr(dut, dictname):
        setattr(dut, dictname, dict())

    debug('Adding dict \"{0}\" to \"{1}\" object'.format(dictname, dut.hostname))
    setattr(dut, dictname, dict_to_attach)


def read_portwd_config(configfile):
    if not path.exists(configfile):
        return None, None

    config = ConfigParser.SafeConfigParser()
    config.optionxform = str
    configbuf = read_local_file(configfile)
    if not configbuf:
        return None, None
    configfp = StringIO.StringIO(configbuf.replace(':', '%%3A'))
    config.readfp(configfp)

    try:
        cable_overrides = dict(config.items('cables'))
    except ConfigParser.NoSectionError:
        cable_overrides = {}
    try:
        port_overrides = dict(config.items('ports'))
    except ConfigParser.NoSectionError:
        port_overrides = {}

    if len(cable_overrides):
        cable_overrides = {k.replace('%%3A', ':'): (v,) for (k, v) in cable_overrides.items()}

    debug('Cable overrides: {}'.format(cable_overrides))
    debug('Port overrides: {}'.format(port_overrides))
    return cable_overrides, port_overrides


def setup_overrides(dut_devs):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        cable_override_dct = {}
        port_override_dct = {}
        configfiles = ['/etc/cumulus/portwd.conf', '/usr/share/cumulus/portwd.conf']
        for configfile in configfiles:
            cable_override, port_override = read_portwd_config(configfile)
            if cable_override:
                cable_override_dct.update(cable_override)
            if port_override:
                port_override_dct.update(port_override)

        dut_obj_install_dict(dut, 'cable_override', cable_override_dct)
        dut_obj_install_dict(dut, 'port_override', port_override_dct)


def convert_parsed_porttab(parsed_porttab_dct, bcm_show_portmap_dct):
    """
    The porttab parser should return a complex dictionary with multiple fields, but we only
    need the mapping swpXX <-> unitZ xeYY.  However, Tomahawk and Trident2+ follow a new set of rules
    So handle both varieties here.
    """
    phy_dct = {}
    for unit in bcm_show_portmap_dct:
        for phy in bcm_show_portmap_dct[unit]:
            logical_port = bcm_show_portmap_dct[unit][phy]['logical']
            phy_dct[logical_port] = (phy, unit)

    porttab_dct = {}
    for port in parsed_porttab_dct:
        sdk_intf = parsed_porttab_dct[port]['sdk_intf']
        logical_port = parsed_porttab_dct[port]['logical_port']
        unit = 'unit' + parsed_porttab_dct[port]['unit']
        if logical_port == '-1' or logical_port == '0' or not logical_port:
            phy = sdk_intf
        else:
            phy, unit = phy_dct[logical_port]

        porttab_dct[port] = (phy, unit)

    return porttab_dct


def dut_install_porttab(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        if bcm_only(dut):
            dut_install_bcm_dct(dut, 'show portmap')
            bcm_show_portmap_dct = dut.bcm_show_portmap_dct
        else:
            bcm_show_portmap_dct = {}
        debug('Collecting /v/l/c/porttab for {}'.format(dut.hostname))
        filename = '/var/lib/cumulus/porttab'
        output = read_local_file(filename)
        if not output:
            critical('No /var/lib/cumulus/porttab.  Cannot continue')
        parsed_porttab_dct = parse('cat ' + filename, output)
        porttab_dct = convert_parsed_porttab(parsed_porttab_dct, bcm_show_portmap_dct)
        debug('Installing porttab_dct to {} with keys {}'.format(
            dut.hostname, porttab_dct.keys()))
        if not porttab_dct:
            critical('porttab_dct is Null')
        dut_obj_install_dict(dut, 'porttab_dct', porttab_dct)

def filter_ports(port):
    port_pattern = re.compile('^sw[0-9]*p[0-9]+(s[0-3])?$')
    if not port_pattern.match(port):
        return False
    return not (os.path.isdir("/sys/class/net/{}/bridge".format(port))
                or os.path.isdir("/sys/class/net/{}/bonding".format(port)))

def dut_install_eeprom_list(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        eeprom_list = []
        asic_vendor = dut.asic_vendor
        if asic_vendor == 'bcm':
            for eeprom_num in list(os.walk("/sys/class/eeprom_dev/"))[0][1]:
                eeprom_name = read_local_file(os.path.join("/sys/class/eeprom_dev", eeprom_num, "label"))
                if 'port' in eeprom_name:
                    eeprom_list.append(eeprom_name.replace('port', ''))

        if asic_vendor == 'mlx':
            eeprom_path = find_path('qsfp1_eeprom', '/sys/devices/')
            debug('eeprom_path: {}'.format(eeprom_path))
            empty_eeproms = []
            for eeprom_info_file in os.listdir(eeprom_path):
                if 'status' in str(eeprom_info_file):
                    file_contents = read_local_file(os.path.join(eeprom_path, eeprom_info_file))
                    if 'not_connected' in file_contents:
                        empty_eeproms.append(str(eeprom_info_file).split('_')[0].split('sfp')[1])
            debug('Empty MLX slots: {}'.format(empty_eeproms))

            for kernel_port in list(os.walk("/sys/class/net/"))[0][1]:
                if not filter_ports(kernel_port):
                    continue 
                if kernel_port.startswith("swp"):
                    kernel_port_num = kernel_port.replace('swp', '').split('s')[0]
                    if kernel_port_num not in empty_eeproms:
                        eeprom_list.append(kernel_port_num)

        eeprom_list = sorted(list(set(eeprom_list)), key=lambda l: int(l))

        debug('Adding eeprom_list: {} to DUT obj: {}'.format(eeprom_list, dut.hostname))
        dut_obj_install_list(dut, 'eeprom_list', eeprom_list)


def dut_install_ethtool_m(dut_devs=None, port_list=None, write_out=False):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    for dut in dut_devs:
        if not hasattr(dut, 'eeprom_list'):
            dut_install_eeprom_list(dut)
        debug('Adding \"ethtool -m\" for each port to DUT obj: {}'.format(dut.hostname))
        ethtool_m_dct = {}
        if not port_list:
            port_list = sorted(dut.porttab_dct, key=lambda l: swp_sort_pattern(l))
        for port in port_list:
            if str(port).replace('swp', '').split('s')[0] in dut.eeprom_list:
                ethtool_m_dct[port] = parse_to_dct(dut, 'ethtool -m {}'.format(port), quiet=True)

        dut_obj_install_dict(dut, 'ethtool_m_dct', ethtool_m_dct)


def dut_install_ethtool(dut_devs=None, port_list=None, write_out=False):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    for dut in dut_devs:
        debug('Adding \"ethtool\" for each port to DUT obj: {}'.format(dut.hostname))
        ethtool_dct = {}
        if not port_list:
            port_list = sorted(dut.porttab_dct, key=lambda l: swp_sort_pattern(l))
        for port in port_list:
            ethtool_dct[port] = parse_to_dct(dut, 'ethtool {}'.format(port), quiet=True)

        dut_obj_install_dict(dut, 'ethtool_dct', ethtool_dct)


def dut_install_os_release(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        debug('Reading /etc/os_release on DUT: {}'.format(dut.hostname))
        os_release = read_local_file('/etc/os-release')

        if not os_release:
            warn('os_release file is empty')

        debug('Adding \"os-release\" {} to DUT obj: {}'.format(os_release, dut.hostname))
        dut_obj_install_string(dut, 'os_release', os_release)


def dut_install_ip_link(dut_devs):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        ip_link_dct = parse_to_dct(dut, 'ip link show')
        dut_obj_install_dict(dut, 'ip_link_dct', ip_link_dct)


def translate_ip_link_status(dut, port):
    if not hasattr(dut, 'ip_link_dct'):
        dut_install_ip_link(dut)

    debug('ip_link flags: {}'.format(dut.ip_link_dct[port]['flags']))
    if dut.ip_link_dct[port]['flags'] == 'BROADCAST,MULTICAST':
        return 'Admin Down', 'Down'
    if dut.ip_link_dct[port]['state'] == 'UP':
        return 'Admin Up', 'Up'
    if dut.ip_link_dct[port]['state'] == 'DOWN':
        return 'Admin Up', 'Down'
    return 'ERROR', 'ERROR'


def dut_install_ethmodes(dut_devs, port_list=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    for dut in dut_devs:
        debug('Adding \"ethmodes\" for each port {} to DUT obj: {}'.format(port_list, dut.hostname))
        ethmode_dct = {}
        if not port_list:
            port_list = sorted(dut.porttab_dct, key=lambda l: swp_sort_pattern(l))
        for port in port_list:
            fuse = '/cumulus/switchd/config/interface/{port}/ethtool_mode'.format(port=port)
            output = read_local_file(fuse)
            if output:
                ethmode_dct[port] = output

            debug('Port: {}, ethmode: {}'.format(port, output))

        dut_obj_install_dict(dut, 'ethmode_dct', ethmode_dct)


def dut_install_eeprom(dut_devs=None, port_list=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    for dut in dut_devs:
        if not hasattr(dut, 'eeprom_list'):
            dut_install_eeprom_list(dut)
        debug('Adding \"eeprom hex output\" for each port {} to DUT obj: {}'.format(port_list, dut.hostname))
        eeprom_dct = {}
        if not port_list:
            port_list = sorted(dut.porttab_dct, key=lambda l: swp_sort_pattern(l))
        for port in port_list:
            if str(port).replace('swp', '').split('s')[0] in dut.eeprom_list:
                eeprom_port_dct = parse_to_dct(dut, "ethtool -m {port} hex on".format(port=port))
                if not eeprom_port_dct:
                    continue
                eeprom_dct[port] = eeprom_port_dct
                debug('Port: {}, EEPROM:'.format(port) + pformat(eeprom_port_dct))
        dut_obj_install_dict(dut, 'eeprom_dct', eeprom_dct)


def dut_install_ethtool_showfec(dut_devs=None, port_list=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    active_fec_pattern = re.compile('^(Active\s)?FEC\sencoding(s)?\s*:\s(?P<active_fec>\w+)$')
    config_fec_pattern = re.compile('Configured FEC encodings:\s(?P<config_fec>\w+)')

    for dut in dut_devs:
        showfec_dct = {}

        debug('Adding \"ethtool --show-fec output\" for each port {} to DUT obj: {}'.format(port_list, dut.hostname))
        if not port_list:
            port_list = sorted(dut.porttab_dct, key=lambda l: swp_sort_pattern(l))
        for port in port_list:
            showfec_dct[port] = {}
            if dut.version[0:1] > '3':
                fuse = '/cumulus/switchd/config/interface/{port}/active_fec'.format(port=port)
            else:
                fuse = '/cumulus/switchd/config/interface/{port}/fec'.format(port=port)
            showfec_dct[port]['hw_active_fec'] = read_local_file(fuse)
            cmd = 'ethtool --show-fec {port}'.format(port=port)
            output = shell_out_catch_errors(cmd)
            if not output:
                continue
            for line in output.split('\n'):
                active_fec_match = active_fec_pattern.match(line)
                if active_fec_match:
                    showfec_dct[port]['active_fec'] = active_fec_match.group('active_fec')
                if dut.version[0:1] > '3':
                    config_fec_match = config_fec_pattern.match(line)
                    if config_fec_match:
                        showfec_dct[port]['config_fec'] = config_fec_match.group('config_fec')

            debug('Dut: {} Port: {}, Active FEC {}, Config FEC: Hw FEC'.format(
                dut.hostname, port,
                showfec_dct[port].get('active_fec', None),
                showfec_dct[port].get('config_fec', None),
                showfec_dct[port].get('hw_active_fec', None)))

        dut_obj_install_dict(dut, 'showfec_dct', showfec_dct)


def get_dom(eeprom):
    if len(eeprom) == 0:
        return {}
    cable = SFFCable(eeprom)
    return cable.dom


def convert_mlx_fec(mlx_fec):
    if 'No FEC' in mlx_fec:
        return 'None'
    elif 'Firecode FEC' in mlx_fec:
        return 'BaseR/FireCode'
    elif 'RS-FEC' in mlx_fec:
        return 'RS'
    elif 'N/A' in mlx_fec:
        return 'None (down)'
    else:
        return mlx_fec


def convert_bcm_fec(bcm_fec):
    if 'No FEC' in bcm_fec:
        return 'None'
    elif 'BaseR' in bcm_fec:
        return 'BaseR/FireCode'
    elif 'N/A' in bcm_fec:
        return 'None (down)'
    else:
        return bcm_fec


def convert_bcm_hw_fec_str(bcm_hw_fec):
    if bcm_hw_fec == 'CL91':
        return 'CL91 (RS)'
    if bcm_hw_fec == 'CL74':
        return 'CL74 (BaseR)'
    return bcm_hw_fec


def convert_bcm_signal_detect(signal_detect):
    try:
        if signal_detect is None or signal_detect == '' or signal_detect == 'N/A':
            return 'NA '
        elif int(signal_detect) == 1:
            return 'Y'
        elif int(signal_detect) == 0:
            return 'N'
        else:
            debug('Unexpected value for signal detect: {}'.format(signal_detect))
            return 'Err'
    except Exception as e:
        debug('Unexpected value for signal detect: {}'.format(e))
        return 'Err'


def get_compliance_codes(eeprom):
    if len(eeprom) == 0:
        return []
    cable = SFFCable(eeprom)
    codes = cable.ethcodes_safe
    return [codes[x] for x in range(0, len(codes))]


def dut_install_topology(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        debug('Collecting topology.dot for {}'.format(dut.hostname))
        filename = '/etc/ptm.d/topology.dot'
        output = read_local_file(filename)
        if output:
            topology_dct = parse('cat ' + filename, output)
        else:
            topology_dct = None

        if not topology_dct:
            debug('topology.dot is Null')
        else:
            debug('Adding topology_dct to object {} with keys {}'.format(dut.hostname, topology_dct.keys()))
        dut_obj_install_dict(dut, 'topology_dct', topology_dct)


def dut_install_lldpctl(dut_devs=None, force_refresh=False):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        refresh = False
        now = time.time()
        if force_refresh:
            refresh = True

        elif not hasattr(dut, 'lldp_dct') or not hasattr(dut, 'lldp_refresh_time'):
            refresh = True

        elif not refresh and now - dut.lldp_refresh_time > 5:
                refresh = True

        if refresh:
            debug('Adding/Refreshing lldp_dct on dut: {}'.format(dut.hostname))
            output = shell_out_catch_errors('lldpctl -f keyvalue')
            lldp_dct = lldp_keyvalue_parser(output)
            dut_obj_install_dict(dut, 'lldp_dct', lldp_dct)
            dut_obj_install_string(dut, 'lldp_refresh_time', now)


def lldp_keyvalue_parser(output):
    lldp_dct = {}
    port_desc = re.compile(r'''lldp\.(swp[0-9s]+)\.port\.descr''')
    chassis_name = re.compile(r'''lldp\.(swp[0-9s]+)\.chassis\.name''')

    for line in output.split('\n'):
        if re.search('/root/.bashrc:', line):
            debug('bashrc error -skip this line')
            continue
        if '=' not in line:
            error('lldpctl line does not contain key.value=data \n {}'.format(line))
            continue

        key, value = line.split('=')
        port_match = port_desc.match(key)
        if port_match:
            local_port = port_match.group(1)
            remote_port = value
            if local_port not in lldp_dct:
                lldp_dct[local_port] = {}
            lldp_dct[local_port]['neighbor_port'] = remote_port
            debug('Local port: {}. neighbor port: {}'.format(local_port, remote_port))
            continue

        chassis_match = chassis_name.match(key)
        if chassis_match:
            local_port = chassis_match.group(1)
            remote_switch = value
            if local_port not in lldp_dct:
                lldp_dct[local_port] = {}
            lldp_dct[local_port]['neighbor_switch'] = remote_switch
            debug('Local port: {}. neighbor switch: {}'.format(local_port, remote_switch))
            continue

    return lldp_dct


def lldp_remote_port_lookup(local_dut, local_port, force_refresh=False):
    dut_install_lldpctl(local_dut, force_refresh=force_refresh)

    neighbor_switch = local_dut.lldp_dct.get(local_port, {}).get('neighbor_switch', None)
    if neighbor_switch:
        neighbor_switch = neighbor_switch.split('.')[0]
    neighbor_port = local_dut.lldp_dct.get(local_port, {}).get('neighbor_port', None)
    debug('LLDP: Dut: {} Port: {}. Neighbor is {} {}'.format(local_dut.hostname, local_port,
                                                             neighbor_switch, neighbor_port))
    return neighbor_switch, neighbor_port


def dut_install_mlx_ports_info(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in mlx_only(dut_devs):
        debug('Adding \"mlx ports info\" to DUT obj: {}'.format(dut.hostname))
        mlx_ports_info_dct = parse_to_dct(dut, '/usr/lib/cumulus/mlxcmd ports info')
        debug('Adding mlx_ports_info_dct to {} with keys {}'.format(
            dut.hostname, mlx_ports_info_dct.keys()))
        if not mlx_ports_info_dct:
            critical('mlx_ports_info_dct is Null')
        dut_obj_install_dict(dut, 'mlx_ports_info_dct', mlx_ports_info_dct)


def is_hurricane2(dut_devs):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    return [dut for dut in dut_devs if dut.asic == 'Hurricane2']


def bcm_only(dut_devs):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    return [dut for dut in dut_devs if dut.asic_vendor == 'bcm']


def mlx_only(dut_devs):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    return [dut for dut in dut_devs if dut.asic_vendor == 'mlx']


def mlxlink_port(port):
    mlx_port = port[3:]
    if 's' in mlx_port:
        mlx_port = mlx_port[0:-2]
    return mlx_port


def dut_install_mlxlink(dut_devs=None, port_list=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    mlxlink_base = (
        "/usr/bin/mlxlink -d $(/usr/bin/mst status | grep pciconf | " +
        "/usr/bin/awk '{{print $1}}') -c -m -e --show_device --show_fec " +
        "-p {mlx_port}")

    for dut in mlx_only(dut_devs):
        mlxlink_dct = {}
        for port in port_list:
            mlx_port = mlxlink_port(port)
            mlxlink_cmd = mlxlink_base.format(mlx_port=mlx_port)
            mlxlink_dct[port] = parse_to_dct(dut, mlxlink_cmd)

        dut_obj_install_dict(dut, 'mlxlink_dct', mlxlink_dct)


def dut_install_version(dut_devs=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    for dut in dut_devs:
        os_version = get_version(dut)
        debug('Adding \"VERSION_ID\" {} to DUT obj: {}'.format(os_version, dut.hostname))
        dut_obj_install_string(dut, 'version', os_version)


def get_version(dut):
    if not hasattr(dut, 'os_release'):
        dut_install_os_release(dut)

    if not dut.os_release:
        return '2.x'

    for line in dut.os_release.split('\n'):
        if 'VERSION_ID' in line:
            return line.split('=')[1]

    warn('Could not determine os-release version')
    return None


def dut_install_bcm_phy_diag_dsc(dut_devs=None, port_list=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]
    if not isinstance(port_list, list):
        port_list = [port_list]

    for dut in bcm_only(dut_devs):
        bcm_phy_diag_dsc_dct = {}
        if not hasattr(dut, 'porttab_dct'):
            dut_install_porttab(dut)
        if dut.version[0:1] < '3':
            continue
        if port_list == ['all']:
            port_list = warp_core_list(dut.porttab_dct)

        debug('Adding Port HW state in \"bcm phy diag X dsc\" to DUT obj: {}'.format(dut.hostname))
        for port in port_list:
            phy, unit = dut.porttab_dct[port]
            bcm_phy_diag_dsc_dct[port] = parse_to_dct(
                dut, '/usr/lib/cumulus/bcmcmd {} phy diag {} dsc'.format(unit[-1:]+':', phy))
            debug('Adding bcm_phy_diag_dsc_dct to {} with keys {}'.format(
                dut.hostname, bcm_phy_diag_dsc_dct.keys()))
            if not bcm_phy_diag_dsc_dct:
                critical('bcm_phy_diag_dsc_dct is Null')

        dut_obj_install_dict(dut, 'bcm_phy_diag_dsc_dct', bcm_phy_diag_dsc_dct)


def warp_core_list(swp_list):
    """For QSFPs, return a list with only the 1st phy in the warp core"""
    sorted_swp_list = sorted(swp_list, key=lambda l: swp_sort_pattern(l))
    if sorted_swp_list[-1][3:5] not in ['32', '16']:
        out_list = []
        for port in sorted_swp_list:
            if int(port[3:5]) in range(1, 48, 4) or (
                    int(port[3:5]) > 48 and port[-2:] not in ['s1', 's2', 's3']):
                out_list.append(port)

    else:
        out_list = []
        for port in sorted_swp_list:
            if port[-2:] not in ['s1', 's2', 's3']:
                out_list.append(port)

    return out_list


def dut_install_bcm_dct(dut_devs=None, cmd=None, dut_obj_property=None):
    if not isinstance(dut_devs, list):
        dut_devs = [dut_devs]

    if not cmd:
        critical('dut_install_bcm_dct called without any command given')
        return

    file_suffix = 'bcm_' + '_'.join(cmd.split())
    if not dut_obj_property:
        dut_obj_property = '_'.join([file_suffix, 'dct'])

    for dut in bcm_only(dut_devs):
        bcm_cmd_dct = dict()
        debug('Adding Port HW state in \"{}\" to DUT obj: {}'.format(cmd, dut.hostname))
        bcm_cmd_dct['unit0'] = parse_to_dct(dut, '/usr/lib/cumulus/bcmcmd 0: {}'.format(cmd), quiet=True)
        if not bcm_cmd_dct['unit0']:
            if 'show counters' in cmd:
                warn('{} is Null'.format(dut_obj_property))
            else:
                error('{} is Null'.format(dut_obj_property))
                error('DUT: {}: bcmcmd {} failed to parse. Some data will be missing.'.format(dut.hostname, cmd))
        else:
            debug('Adding unit0 {} to {} with keys {}'.format(
                dut_obj_property, dut.hostname, bcm_cmd_dct['unit0'].keys()))

        if is_hurricane2(dut):
            bcm_cmd_dct['unit1'] = parse_to_dct(dut, '/usr/lib/cumulus/bcmcmd 1: {}'.format(cmd), quiet=True)
            debug('Adding unit1 {} to {} with keys {}'.format(
                dut_obj_property, dut.hostname, bcm_cmd_dct.keys()))
            if not bcm_cmd_dct['unit1']:
                if 'show counters' in cmd:
                    warn('Unit1: {} is Null'.format(dut_obj_property))
                else:
                    critical('DUT: {}. Unit1: {} is Null'.format(dut_obj_property), dut.hostname)

        dut_obj_install_dict(dut, dut_obj_property, bcm_cmd_dct)


